//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  DeepSkyBlue
#property indicator_color2  PaleVioletRed
#property indicator_color3  Silver
#property indicator_width1  2
#property indicator_width2  2
#property indicator_minimum 0
#property indicator_maximum 1

//
//
//
//
//

extern string TimeFrame1   = "Current time frame";
extern string TimeFrame2   = "Next time frame";
extern int    SmoothFactor = 5;
extern int    RSIPeriod    = 14;
extern double WP           = 4.236;

//
//
//
//
//

double qqeUp[];
double qqeDn[];
double qqeNe[];

//
//
//
//
//

string indicatorFileName;
bool   calculateValue;
bool   returnBars;
bool   timeFrame1;
bool   timeFrame2;

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,qqeUp); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,qqeDn); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,qqeNe); SetIndexStyle(2,DRAW_HISTOGRAM);


   //
   //
   //
   //
   //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame1=="returnBars";     if (returnBars)     return(0);
      calculateValue    = TimeFrame1=="calculateValue"; if (calculateValue) return(0);
      timeFrame1        = stringToTimeFrame(TimeFrame1);
         if (StringFind(StringUpperCase(TimeFrame2),"NEXT")>-1)
               timeFrame2 = nextTimeFrame(timeFrame1);
         else  timeFrame2 = stringToTimeFrame(TimeFrame2);

   //
   //
   //
   //
   //
         
   IndicatorShortName("QQE combined "+timeFrameToString(timeFrame1)+" - "+timeFrameToString(timeFrame2)+" ("+SmoothFactor+","+RSIPeriod+")");
   return(0);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//

double work[][5];
#define _ema1  0
#define _ema2  1
#define _tr    2
#define _rsi   3
#define _trend 4


//
//
//
//
//

int start()
{
   int i,r,limit,counted_bars=IndicatorCounted();
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = Bars-counted_bars;
           if (returnBars) { qqeUp[0] = limit+1; return(0); }
           if (ArrayRange(work,0) != Bars) ArrayResize(work,Bars);

   //
   //
   //
   //
   //

   if (calculateValue)
   {
      double alpha1 = 2.0/(SmoothFactor+1.0);
      double alpha2 = 1.0/(RSIPeriod);
      for (i=limit, r=Bars-i-1; i>=0; i--,r++)
      {  
         work[r][_rsi]  = work[r-1][_rsi]  + alpha1*(iRSI(NULL,0,RSIPeriod,PRICE_CLOSE,i)   - work[r-1][_rsi] );
         work[r][_ema1] = work[r-1][_ema1] + alpha2*(MathAbs(work[r-1][_rsi]-work[r][_rsi]) - work[r-1][_ema1]);
         work[r][_ema2] = work[r-1][_ema2] + alpha2*(work[r][_ema1]                         - work[r-1][_ema2]);

         //
         //
         //
         //
         //

         double dar = work[r  ][_ema2]*WP;
         double tr  = work[r-1][_tr];
         double dv  = tr;
   
            if (work[r][_rsi]   < tr) { tr = work[r][_rsi] + dar; if ((work[r-1][_rsi] < dv) && (tr > dv)) tr = dv; }
            if (work[r][_rsi]   > tr) { tr = work[r][_rsi] - dar; if ((work[r-1][_rsi] > dv) && (tr < dv)) tr = dv; }
                work[r][_tr]    = tr;
                work[r][_trend] = work[r-1][_trend];
         
                if (work[r][_tr]<work[r][_rsi]) work[r][_trend] =  1;
                if (work[r][_tr]>work[r][_rsi]) work[r][_trend] = -1;
                qqeUp[i] = work[r][_trend];
      }    
      return(0);
   }      

   //
   //
   //
   //
   //
       
   if (timeFrame1 != Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame1,indicatorFileName,"returnBars",0,0)*timeFrame1/Period()));
   if (timeFrame2 != Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame2,indicatorFileName,"returnBars",0,0)*timeFrame2/Period()));
   for(i=limit; i>=0; i--)
   {
      double trend  = iCustom(NULL,timeFrame1,indicatorFileName,"calculateValue","",SmoothFactor,RSIPeriod,WP,0,iBarShift(NULL,timeFrame1,Time[i]));
             trend += iCustom(NULL,timeFrame2,indicatorFileName,"calculateValue","",SmoothFactor,RSIPeriod,WP,0,iBarShift(NULL,timeFrame2,Time[i]));
         
         //
         //
         //
         //
         //
         
         qqeUp[i] = EMPTY_VALUE;
         qqeDn[i] = EMPTY_VALUE;
         qqeNe[i] = EMPTY_VALUE;
            if (trend>0) { qqeUp[i] = 1; continue; }
            if (trend<0) { qqeDn[i] = 1; continue; }
                           qqeNe[i] = 1;
   }       
   return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int nextTimeFrame(int tf)
{
   double max = ArraySize(iTfTable)-1;
   int    ret = -1;
   
      //
      //
      //
      //
      //
      
      for (int i=max; i>=0 && tf!=iTfTable[i]; i--) ret = iTfTable[i];
   return(MathMax(ret,tf));
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}